
#ifndef QOS_MGR_TYPES_H
#define QOS_MGR_TYPES_H

/*!
 * defines req_ele_t
 */
typedef struct qos_request_element_struct req_ele_t;

/*!
 * defines qos_tree_ptr_t
 */
typedef struct qos_tree_node* qos_tree_ptr_t;

/*!
 * definition of the error code used throughout the project
 */
#define E_ERROR -1

/*!
 * \brief  format of a qos reservation number
 *
 * format of a qos reservation number \n
 * app_id is the Client-ID of the application which uses the resources reserved during this reservation cycle. \n
 * res_count is used to distinguish between multiple reservations made by a single application .\n
 * alt_count is used during the pre-reserve step to distinguish different alternatives. \n
 * From the viewpoint of a qos_rmgr requests with equal app_id, res_id, alt_count values belong to the same alternative. \n
 * The sum of these requests must be reserved (if this alternative is chosen) during the reserve step. \n
 * Requests with equal app_id, res_id, and differing alt_count values belong to different alternatives. It is sufficient to 
 * pre-reserve the value of the alternative with the highest request to allow a successful reservation for any of the 
 * alternatives. \n
 * acc_count is used during the reserve_step to return the correct handles for the requests:
 * for example: "mem" has pre_reserved for the same alternative: acc_count=1 -> 5 "units"; acc_count=2 4 "units"
 * These resources were being requested by different callers (e.g. intermediate managers). \n
 * Different handles must be generated for each of these request and returned when called for.
 */
struct qos_res_nr
{
  ClientID app_id;
  long     res_count;
  long     alt_count;
  long     acc_count;
};

/*!
 * defines qos_tree_ptr_t
 */
typedef struct qos_res_nr qos_res_nr_t;
  
/*!
 * definition of QOS_INVALID_RNR; used whenever the current value of a reservation nr should be considered not valid
 */
#define QOS_INVALID_RNR ((qos_res_nr_t)INVALID_CLIENT_ID)

#endif //QOS_MGR_TYPES_H
